#include <stdio.h>
#include <stdlib.h>

  /* Vezane liste - Primjer 9a.
     Sortiranje liste MergeSort algoritmom, uz poznati broj elemenata.
     Raspolavljanje liste koristenjem broja elemenata i pokazivaca.
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

lista merge(lista prvi_1, lista prvi_2)
{
        /* Sortirano spaja dvije liste (merge). */

    lista prvi = NULL, zadnji, pom;

        /* Ako je jedna lista prazna, rezultat je ona druga lista. */
    if (prvi_1 == NULL) return prvi_2;
    if (prvi_2 == NULL) return prvi_1;

        /* U nastavku obrade, obje liste sigurno NISU prazne. */

        /* Prolaz po obje liste, sve dok su obje neprazne. */
    while (prvi_1 != NULL && prvi_2 != NULL) {

            /* Nadji manjeg, od prvog iz prve liste i prvog iz
               druge liste. Izbaci ga s pocetka njegove liste,
               tj. skrati pripadnu listu. */
        if (prvi_1->broj <= prvi_2->broj) {
            pom = prvi_1;
            prvi_1 = prvi_1->sljed;
        }
        else {
            pom = prvi_2;
            prvi_2 = prvi_2->sljed;
        }

            /* Ubaci ga na kraj sortirane liste. */
        if (prvi == NULL)
            prvi = pom;
        else
            zadnji->sljed = pom;
        zadnji = pom;
    }

        /* Spoji neprazni ostatak (tocno jedna lista, druga je prazna)
           na kraj sortirane liste. */
    if (prvi_1 == NULL)
        zadnji->sljed = prvi_2;

    if (prvi_2 == NULL)
        zadnji->sljed = prvi_1;

    return prvi;
}

/* ======================================================================== */

lista merge_sort_n(lista prvi, int n)
{
        /* Sortira listu Merge_Sort algoritmom.
           Poznato je da lista ima n elemenata. */

    int m, nm, i;
    lista zadnji, prvi_2;

        /* Test na praznu ili jednoclanu listu, preko
           broja elemenata, a ne preko pokazivaca. */
    if (n <= 1)
        return prvi;

        /* U nastavku obrade, lista ima bar dva elementa. */

        /* Raspolovimo broj elemenata. Ako je n neparan,
           onda je m = n/2 < n - m (u protivnom su isti).
           Broj elemenata u prvom dijelu liste je nm = n - m,
           zato da prvi dio bude dulji, ako je n neparan. */
    m = n >> 1;    /* = n / 2. */
    nm = n - m;

        /* ``Raspolovi'' listu. Pomicemo pokazivac zadnji za
           nm - 1 mjesta od prvog. Nakon toga, on pokazuje na
           zadnjeg u prvom dijelu. */
    zadnji = prvi;
    for (i = 1; i < nm; ++i)
        zadnji = zadnji->sljed;

        /* Pokazivac zadnji sad korektno pokazuje na zadnjeg u prvom
           dijelu. Pokazivac prvi_2 postavljamo na prvog u drugom
           dijelu (prvi iza zadnjeg) i korektno zavrsavamo prvi dio. */
    prvi_2 = zadnji->sljed;
    zadnji->sljed = NULL;

        /* Rekurzivno sortiranje obje liste i sortirano spajanje. */
    prvi = merge(merge_sort_n(prvi, nm),
                 merge_sort_n(prvi_2, m));

    return prvi;
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi = NULL;    /* Prazna lista! */
    lista zadnji;
    int broj, n;

        /* Punjenje liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi = kreiraj_straga(prvi, &zadnji, broj);
    } while (broj != 0);

        /* Ispis broja elemenata i cijele liste. */

    printf("\n Polazna lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Sortiranje liste - mergesort,
           s poznatim brojem elemenata. */

    n = broj_elemenata(prvi);
    prvi = merge_sort_n(prvi, n);

        /* Sortirana lista:
           ispis broja elemenata i cijele liste. */

    printf("\n Sortirana lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
